#
# Copyright (c) 2008 Tilman Sauerbeck (tilman at code-monkey de)
#
# Permission is hereby granted, free of charge, to any person obtaining
# a copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be
# included in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
# EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE
# LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION
# OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
# WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#

require 'libmvi.rb'
require 'libpalette.rb'
require 'imlib2'

case ARGV.shift
when "dump"
	mvi = Mvi::File.new(ARGV.shift)

	Dir.chdir(ARGV.shift) do
		mvi.items.each_with_index do |item, i|
			File.open("chunk%02i.dat" % i, "wb") do |f|
				f.write(item.body)
			end
		end
	end
when "info"
	mvi = Mvi::File.new(ARGV.shift)

	mvi.items.each_with_index do |item, i|
		puts "[#{i}] #{item.channel}, #{item.type}, #{item.length} bytes"
	end
when "decode"
	mvi = Mvi::File.new(ARGV.shift)
	dest = ARGV.shift

	palette = nil

	left = File.open("/tmp/left.pcm", "w")
	right = File.open("/tmp/right.pcm", "w")

	0.upto(mvi.items.length - 1) do |i|
		item = mvi.items[i]

		# FIXME: this is wrong, but it works.
		# the "type" method needs to be renamed to something better,
		# just like the values that it returns.
		case item.type
		when :colormap
			puts "got colormap on channel #{item.channel}"

			case item.channel
			when 5
				palette = Palette.from_buf(mvi.items[i].body)
			when 3
				left.write(mvi.items[i].body)
			when 4
				right.write(mvi.items[i].body)
			end

			next
		when :audio
			puts "got 'audio' on channel #{item.channel}"
			next
		when :video
			#puts "got 'video' on channel #{item.channel}"
			#next
		end

		# get the pixel data
		refs = item.decode
		file = File.join(dest, "%04i.png" % i)

		image = Imlib2::Image.new(mvi.header.video_width,
		                          mvi.header.video_height)

		data = " " * (image.width * image.height * 4)
		offset = 0
		count = 0
		pitch = mvi.header.video_width * 4

		# some videos only store every 2nd pixel (that's the large ones,
		# ie cut scenes, approach movies and dock movies)
		loop_count = image.width * image.height / refs.length

		refs.each do |ref|
			raise "< 0" if ref < 0
			raise " > 255" if ref > 255

			# ask the palette for the color at the given index
			pixel = palette.color(ref)

			# dump the pixel(s) to the image
			loop_count.times do
				data[offset + 0] = pixel[2]
				data[offset + 1] = pixel[1]
				data[offset + 2] = pixel[0]
				data[offset + 3] = 0xff
				offset += 4
			end
		end

		# save the image to disk
		image.data = data
		image.save(file)
	end
else
	STDERR.puts <<EOF
usage: decode_mvi.rb COMMAND input output

COMMAND is either "dump" or "info" or "decode".
EOF
end
